#ifndef __SETTINGS_H__
# define __SETTINGS_H__

#include <num/num.h>
using namespace libnum;

/* Radius of the level cylinder, in world units. */
#define LEVEL_RADIUS num(4.0)
/* Radius of the kill plane */
#define KILL_PLANE_RADIUS num(10.0)
/* Section lengths, in world units.
   TODO: Remove, we don't use sections anymore */
#define SECTION_LENGTH num(3.0)
/* Number of platforms in the world cylinder. */
#define PLATFORM_COUNT 10
/* Platforms len information, in word units */
#define PLATFORM_LEN_LONG   num(3.0)
#define PLATFORM_LEN_SHORT  num(1)
/* Magnitude of the gravity field, locally (world units/s^2), when holding the
   jump button and when releasing it. */
#define HOVERING_GRAVITY num(-1.25)
#define FALLING_GRAVITY num(-8.0)
/* Vertical speed granted when a jump starts (world units/s). */
#define JUMP_THRUST num(1.5)
/* Maximum height of a step that can be crossed without dying. */
#define STEP_HEIGHT (LEVEL_RADIUS * num(0.09))
/* Player height, for collisions with platform fronts. */
#define PLAYER_HEIGHT num(0.8)

/* Duration of a rotation by one platform, in seconds. */
#define TIME_ROTATION num(0.1)
/* How long a jump must last for (s). */
#define TIME_JUMP_THRUST_MIN num(0.25)

/* Vertical FOV, in degrees */
#define RENDER_FOV 120.0
/* Height of camera above the player's feet, in world units */
#define RENDER_EYE_HEIGHT num(1.0)
/* Number of sections visible in advance */
#define RENDER_DISTANCE 8
/* Distance between the player and the camera */
#define RENDER_CAMERA_BACK_DISTANCE num(0.67)
/* Depth at which we start rendering platforms */
#define RENDER_DEPTH (num(8.0) * SECTION_LENGTH)

/* Set to 1 to enable logging by USB. */
#define LOG_USB_ENABLE 0

#endif /* __SETTINGS_H__ */
